#ifndef __ATTRIBUT_REGISTRAR__
#define __ATTRIBUT_REGISTRAR__

#include "attribut.hpp"

using XMLIOSERVER::XML::THashAttributes;

namespace XMLIOSERVER
{
   class AttributRegistrar
   {
      protected :

         AttributRegistrar(void)
            : attrList()
         { /* Ne rien faire de plus */ }

         AttributRegistrar(const AttributRegistrar& _ar)
            : attrList(_ar.attrList)
         { /* Ne rien faire de plus */ }

         void RegisterAttribut(BaseAttribut* const attribut){ attrList.addObject(attribut); }

         const StrHashMap<BaseAttribut>& getAttributList(void) const { return (attrList); }

         size_t getNbAttributes(void) const { return (attrList.getSize()); }

         bool hasAttribut(const string& _id) const { return (attrList.hasMappedValue(_id)); }

         const StrHashMap<BaseAttribut>& getAttrList(void) const { return (attrList); }

         BaseAttribut* getAttribut(const string& _id) throw (XMLIOUndefinedValueException) { return (attrList[_id]); }

         friend ostream& operator<< (ostream& out, const AttributRegistrar& c)
         {
            for(unsigned int i = 0; i < c.attrList.getVectorSize(); i++)
               if (c.attrList.getVector()[i]->hasValue())
                  out << *c.attrList.getVector()[i];
            return (out);
         }

         void setAttributes(const THashAttributes& _attr)
         {
            for (THashAttributes::ConstIterator it = _attr.begin(); it != _attr.end(); it++)
               if ((*it).first.compare(string("id"))!= 0 and (*it).first.compare(string("src"))!=0)
               // (Au dessus) Non prise en compte de l'identifiant et de l'inclusion de fichiers externes lors de l'affectation des attributs.
                  this->setSAttribut((*it).first, (*it).second);
            return;
         }

         void addAttributes(const AttributRegistrar& _parent)
         {
            const StrHashMap<BaseAttribut>& _pattr = _parent.getAttributList();
            const StrHashMap<BaseAttribut>& _lattr = getAttributList();
            //_pattr contient les attributs du parent, _lattr les attributs locaux.

            for(unsigned int i = 0; i < _lattr.getVectorSize(); i++)
            {
               if(_lattr.getVector()[i]->hasValue() or !_pattr.getVector()[i]->hasValue()) continue;
               _lattr.getVector()[i]->assignValue(_pattr.getVector()[i]);
            }
         }

         void setSAttribut(const string& att_name, const std::string& value)
         {
            if (hasAttribut(att_name)) getAttribut(att_name)->setFromString(value);
            else throw XMLIOUndefinedValueException("Impossible de trouver l'attribut nommé \""
               + att_name +"\" dans la liste des attributs enregistrés !");
         }

        ~AttributRegistrar(void)
        {/* Ne rien faire de plus */}

      private :

         StrHashMap<BaseAttribut> attrList;


   }; // class AttributRegistrar
} // namespace XMLIOSERVER


#endif //__ATTRIBUT_REGISTRAR__
